//
//  AQPlayFileController.m
//  AudioQueuePlayFile
//
//  Copyright 2011 JG Electronics. All rights reserved.
//	Remarks, reports to: www.jgelectronics.nl.
//
//	Derived from Apple's example program AudioQueueTools.
//	UI and icons from MPlayerX an open source application by http://mplayerx.org
//
//	OSX10.6 and XCODE 4.0.1
//
/*
 Disclaimer: IMPORTANT: 
 This JG Electronics software is supplied to you by JG Electronics as FREE AND OPEN SOURCE SOFTWARE. 
 
 The JG Electronics Software is provided by JG Electronics on an "AS IS" basis.
 JG ELECTRONICS MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE JG ELECTRONICS SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL JG ELECTRONICS BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE JG ELECTRONICS SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF JG ELECTRONICS HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

#import "AQPlayFileController.h"

@implementation AQPlayFileController

	#pragma mark -
	#pragma mark === Accessor Methods ===
//	*********************************************************************************************************************************
//	archive strings
	@synthesize pfFileNameStr;
//	slider values
	@synthesize pfAmplitudeFlt;

	#pragma mark -
	#pragma mark === Internal Methods ===
//	*********************************************************************************************************************************
- (id)init
{
    self = [super init];
    if (self) {
	//	pfPlayFileModelPtr = [[AQPlayFileModel2 alloc] init];
		pfPlayFileModelPtr = [AQPlayFileModel2 new];
	//
		pfFileNameStr = @"";
		[self setPfAmplitudeFlt:0.75];
		[self pfSetMuteButtonImage];
		pfAmplitudeOldFlt = 0.75;
		pfFileLoaded = NO;
		pfFileInProgress = NO;
		
		nc = [NSNotificationCenter defaultCenter];
		[nc addObserver:self selector:@selector(pfPlayBackStopNotification:) name:@"PlayBackStop" object:nil];
		
	//	key value observing
		[self addObserver:self forKeyPath:@"pfAmplitudeFlt" options:NSKeyValueObservingOptionOld context:nil];
    }
    
    return self;
}

- (void)dealloc
{
    [super dealloc];
	[pfFileNameStr release];
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}

	#pragma mark -
	#pragma mark === Key Value Observing Methods ===
//	*********************************************************************************************************************************
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
//	NSLog(@"observeValueForKeyPath");
	if([keyPath compare:@"pfAmplitudeFlt"] == NSOrderedSame) {
		[pfPlayFileModelPtr pfSetamplitude:[self pfAmplitudeFlt]];
		[self pfSetMuteButtonImage];
	}
}

	#pragma mark -
	#pragma mark === Notification Methods ===
//	*********************************************************************************************************************************
- (void)pfPlayBackStopNotification:(NSNotification *)notification {
	NSLog(@"pfPlayBackStopNotification Start");
	BOOL succes = [pfPlayFileModelPtr pfStopPlayBack];
	if(succes == YES) {
		pfFileInProgress = NO;
		pfFileLoaded = NO;
		[nc postNotificationName:@"titleStrNotification" object:@"no audio file"];
		[pfStartPlayBack setState:NSOffState];
	} else {
		NSLog(@"Stop Recording failed");
	}
}

	#pragma mark -
	#pragma mark === Action Methods ===
//	*********************************************************************************************************************************
- (IBAction)pfPlayPause:(id)sender {
	NSLog(@"pfPlayPause");
	BOOL	succes = YES;
	
	if([sender state] == NSOnState) {
		NSLog(@"pfPlay");
		if((pfFileInProgress == NO)&&(pfFileLoaded == YES)) {
			pfFileInProgress = YES;
			NSLog(@"pfFilePathStr: %@", pfFileNameStr);
			NSURL *	soundFileURL = [NSURL fileURLWithPath:pfFileNameStr];
			succes = [pfPlayFileModelPtr pfConfigureAudioFile:(CFURLRef)soundFileURL];
			if(succes == NO) {
				NSLog(@"ConfigureAudioFile failed");
			}
			succes = [pfPlayFileModelPtr pfStartPlayBack];
			if(succes == YES) {
				NSLog(@"Playback Started");
			} else {
				NSLog(@"Playback Start failed");
			}
		} else if(pfFileInProgress == YES) {
		//	restart from pause
		}
	} else if(pfFileInProgress == YES) {
		//	pause
	}
}


- (IBAction)pfMuteaudio:(id)sender {
	if([sender state] == NSOnState) {	
		pfAmplitudeOldFlt = [self pfAmplitudeFlt];
		[self setPfAmplitudeFlt:0.0];
		[pfMute setImage:[NSImage imageNamed:@"mute"]];
	} else {
		[self setPfAmplitudeFlt:pfAmplitudeOldFlt];
		[self pfSetMuteButtonImage];
	}
}


- (IBAction)pfFileBegin:(id)sender {
	NSLog(@"pfFileBegin");
}


- (IBAction)pfFileEnd:(id)sender {
	NSLog(@"pfFileEnd");

	[nc postNotificationName:@"PlayBackStop" object:nil];	
}

	#pragma mark -
	#pragma mark === Button and Slider Image Methods ===
//	*********************************************************************************************************************************
- (void)pfSetMuteButtonImage {
	int index = 100 * [self pfAmplitudeFlt];
	
	if(index < 6) {
		[pfMute setImage:[NSImage imageNamed:@"vol_no"]];
	}
	if((index > 5)&&(index < 36)) {
		[pfMute setImage:[NSImage imageNamed:@"vol_low"]];
	}
	if((index > 35)&&(index < 66)) {
		[pfMute setImage:[NSImage imageNamed:@"vol_mid"]];
	}
	if(index > 65) {
		[pfMute setImage:[NSImage imageNamed:@"vol_high"]];
	}
}

	#pragma mark -
	#pragma mark === Data Archive Methods ===
//	*********************************************************************************************************************************
- (IBAction)pfOpenSoundFile:(id)sender {
    NSOpenPanel*	openPanel = [NSOpenPanel openPanel];
	NSArray *		fileTypes = [NSArray arrayWithObjects:@"m4a", @"mp3", @"aiff", nil];
	
	[openPanel setAllowedFileTypes:fileTypes];
    [openPanel setAllowsMultipleSelection:NO];
//	[openPanel beginSheetModalForWindow:pfWindow completionHandler:^(NSInteger result) {
	[openPanel beginWithCompletionHandler:^(NSInteger result) {
        if (result == NSOKButton) {
            [openPanel orderOut:self];
			NSArray *	fileURLsToOpen = [openPanel URLs];
			[self setPfFileNameStr:[[fileURLsToOpen objectAtIndex:0]path]];
			NSLog(@"Open Panel Returned: %@.\n", pfFileNameStr);
			[nc postNotificationName:@"titleStrNotification" object:pfFileNameStr];
			[pfStartPlayBack setState:NSOffState];
			pfFileLoaded = YES;
		}
    }];
}

@end
