//
//  TitleView.m
//  QT Test
//
//  Created by John Geelen on 16-07-11.
//
//  Copyright 2011 JG Electronics. All rights reserved.
//	Remarks, reports to: www.jgelectronics.nl.
//
//	Derived from Apple's example program AudioQueueTools.
//	UI and icons from MPlayerX an open source application by http://mplayerx.org
//
//	OSX10.6 and XCODE 4.0.1
//
/*
 Disclaimer: IMPORTANT: 
 This JG Electronics software is supplied to you by JG Electronics as FREE AND OPEN SOURCE SOFTWARE. 
 
 The JG Electronics Software is provided by JG Electronics on an "AS IS" basis.
 JG ELECTRONICS MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE JG ELECTRONICS SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL JG ELECTRONICS BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE JG ELECTRONICS SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF JG ELECTRONICS HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

#import "TitleView.h"

static NSString * const kStringDots = @"...";
static NSRect trackRect;

@implementation TitleView

	#pragma mark -
	#pragma mark === Accessor Methods ===
//	*********************************************************************************************************************************
//	strings
@synthesize titleStr;

	#pragma mark -
	#pragma mark === Internal Methods ===
//	*********************************************************************************************************************************
-(void) awakeFromNib {
	[self addSubview:closeButton];
	[self addSubview:miniButton];
	[self addSubview:zoomButton];
	
	[closeButton setTarget:[self window]];
	[miniButton setTarget:[self window]];
	[zoomButton setTarget:[self window]];
	
	[closeButton setAction:@selector(performClose:)];
	[miniButton setAction:@selector(performMiniaturize:)];
	[zoomButton setAction:@selector(performZoom:)];
	
		//	notifications
	NSNotificationCenter *	nc = [NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(setTitleStrNotification:) name:@"titleStrNotification" object:nil];
		//	because Appdelegate is BeaseWindow's delegate we listen for window's notifications
	[nc addObserver:self selector:@selector(windowDidBecomKey:) name:NSWindowDidBecomeKeyNotification object:[self window]];
	[nc addObserver:self selector:@selector(windowDidResignKey:) name:NSWindowDidResignKeyNotification object:[self window]];
}

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        titleStr = @"";
		titleAttr = [[NSDictionary alloc]
					 initWithObjectsAndKeys:
					 [NSColor whiteColor], NSForegroundColorAttributeName,
					 [NSFont titleBarFontOfSize:0], NSFontAttributeName,
					 nil];
		
		tbCornerLeft	= [[NSImage imageNamed:@"titlebar-corner-left.png"] retain];
		tbCornerRight	= [[NSImage imageNamed:@"titlebar-corner-right.png"] retain];
		tbMiddle		= [[NSImage imageNamed:@"titlebar-middle.png"] retain];
		
		imgCloseActive	 = [[NSImage imageNamed:@"close-active.tiff"] retain];
		imgCloseInactive = [[NSImage imageNamed:@"close-inactive-disabled.tiff"] retain];
		imgCloseRollover = [[NSImage imageNamed:@"close-rollover.tiff"] retain];
		
		imgMiniActive	 = [[NSImage imageNamed:@"minimize-active.tiff"] retain];
		imgMiniInactive	 = [[NSImage imageNamed:@"minimize-inactive-disabled.tiff"] retain];
		imgMiniRollover	 = [[NSImage imageNamed:@"minimize-rollover.tiff"] retain];
		
		imgZoomActive	 = [[NSImage imageNamed:@"zoom-active.tiff"] retain];
		imgZoomInactive	 = [[NSImage imageNamed:@"zoom-inactive-disabled.tiff"] retain];
		imgZoomRollover	 = [[NSImage imageNamed:@"zoom-rollover.tiff"] retain];
		
		closeButton = [[NSButton alloc] initWithFrame:NSMakeRect( 4, 0, 22, 22)];
		miniButton  = [[NSButton alloc] initWithFrame:NSMakeRect(25, 0, 22, 22)];
		zoomButton  = [[NSButton alloc] initWithFrame:NSMakeRect(46, 0, 22, 22)];
		
		[closeButton setButtonType:NSSwitchButton];
		[closeButton setImage:imgCloseActive];
		[closeButton setImagePosition:NSImageOnly];
		[closeButton setBordered:NO];		
		[closeButton setAutoresizingMask:NSViewMaxXMargin|NSViewMaxYMargin];
		[closeButton setContinuous:NO];
		
		[miniButton setButtonType:NSSwitchButton];
		[miniButton setImage:imgMiniActive];
		[miniButton setImagePosition:NSImageOnly];
		[miniButton setBordered:NO];
		[miniButton setAutoresizingMask:NSViewMaxXMargin|NSViewMaxYMargin];
		[miniButton setContinuous:NO];
		
		[zoomButton setButtonType:NSSwitchButton];
		[zoomButton setImage:imgZoomInactive];
		[zoomButton setImagePosition:NSImageOnly];
		[zoomButton setBordered:NO];
		[zoomButton setAutoresizingMask:NSViewMaxXMargin|NSViewMaxYMargin];
		[zoomButton setContinuous:NO];
		
		mouseEntered = NO;
    }
    return self;
}

- (void)dealloc {
	[titleStr release];
	[titleAttr release];
	
	[closeButton release];
	[miniButton release];
	[zoomButton release];
	
	[tbCornerLeft release];
	[tbCornerRight release];
	[tbMiddle release];
	
	[imgCloseActive release];
	[imgCloseInactive release];
	[imgCloseRollover release];
	
	[imgMiniActive release];
	[imgMiniInactive release];
	[imgMiniRollover release];
	
	[imgZoomActive release];
	[imgZoomInactive release];
	[imgZoomRollover release];
	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	
    [super dealloc];
}

	#pragma mark -
	#pragma mark === Notification Methods ===
//	*********************************************************************************************************************************
- (void)setTitleStrNotification:(NSNotification *)notification {
//	NSLog(@"setTitleStrNotification Start");

	NSString *	title = [notification object];
	[self setTitleStr: title];
	[self setNeedsDisplay:YES];
}

	#pragma mark -
	#pragma mark === View Methods ===
//	*********************************************************************************************************************************
-(void) windowDidBecomKey:(NSNotification*) notif
{
	[closeButton setImage:imgCloseActive];
	[miniButton setImage:imgMiniActive];
	[zoomButton setImage:imgZoomInactive];
}

-(void) windowDidResignKey:(NSNotification*) notif
{
	[closeButton setImage:imgCloseInactive];
	[miniButton setImage:imgMiniInactive];
	[zoomButton setImage:imgZoomInactive];
}

-(BOOL) acceptsFirstMouse:(NSEvent *)event {
	return YES;
}

-(BOOL) acceptsFirstResponder {
	return YES;
}

-(void) mouseUp:(NSEvent *)theEvent
{
	if ([theEvent clickCount] == 2) {
		[[self window] performMiniaturize:self];
	}
}

-(void) mouseMoved:(NSEvent *)theEvent
{
	BOOL mouseIn = NSPointInRect([self convertPoint:[theEvent locationInWindow] fromView:nil], trackRect);
	
	if (mouseIn != mouseEntered) {
		mouseEntered = mouseIn;
	// entered
		if (mouseEntered) {
			[closeButton setImage:imgCloseRollover];
			[miniButton setImage:imgMiniRollover];
			[zoomButton setImage:imgZoomInactive];			
		} else {
		// exited
			if ([[self window] isKeyWindow]) {
				[closeButton setImage:imgCloseActive];
				[miniButton setImage:imgMiniActive];
				[zoomButton setImage:imgZoomInactive];
			} else {
				[closeButton setImage:imgCloseInactive];
				[miniButton setImage:imgMiniInactive];
				[zoomButton setImage:imgZoomInactive];				
			}
		}
	}
}

	#pragma mark -
	#pragma mark === Drawing Methods ===
//	*********************************************************************************************************************************
- (void)drawRect:(NSRect)dirtyRect
{	
	NSSize leftSize = [tbCornerLeft size];
	NSSize rightSize = [tbCornerRight size];
	NSSize titleSize = [self bounds].size;
	NSPoint drawPos;
	
	drawPos.x = 0;
	drawPos.y = 0;
	
	dirtyRect.origin.x = 0;
	dirtyRect.origin.y = 0;
	
	dirtyRect.size = leftSize;
	[tbCornerLeft drawAtPoint:drawPos fromRect:dirtyRect operation:NSCompositeCopy fraction:1.0];
	
	drawPos.x = titleSize.width - rightSize.width;
	dirtyRect.size = rightSize;
	[tbCornerRight drawAtPoint:drawPos fromRect:dirtyRect operation:NSCompositeCopy fraction:1.0];
	
	dirtyRect.size = [tbMiddle size];
	[tbMiddle drawInRect:NSMakeRect(leftSize.width, 0, titleSize.width-leftSize.width-rightSize.width, titleSize.height)
				fromRect:dirtyRect
			   operation:NSCompositeCopy
				fraction:1.0];
	
	if (titleStr) {
		NSMutableString *renderStr = [titleStr mutableCopy];
		NSSize dotSize = [kStringDots sizeWithAttributes:titleAttr];
		NSSize strSize = [renderStr sizeWithAttributes:titleAttr];
		float widthMax = titleSize.width - 80;
		
		if (strSize.width > widthMax) {
	//	the title less than 3 characters should be never longer than widMax,
	//	so it is safe to delete the first three chars, without checking
			[renderStr deleteCharactersInRange:NSMakeRange(0, 2)];
			
			while (dotSize.width + strSize.width > widthMax) {
				[renderStr deleteCharactersInRange:NSMakeRange(0, 1)];
				strSize = [renderStr sizeWithAttributes:titleAttr];
			}
			[renderStr insertString:kStringDots	atIndex:0];
		}
		
		dirtyRect.size = [renderStr sizeWithAttributes:titleAttr];
		
		drawPos.x = MAX(70, (titleSize.width -dirtyRect.size.width)/2);
		drawPos.y = (titleSize.height - dirtyRect.size.height)/2;
		
		[renderStr drawAtPoint:drawPos withAttributes:titleAttr];
		[renderStr release];
	}
}

@end
