//
//  UIView.m
//  QT Test
//
//  Created by John Geelen on 16-07-11.
//
//  Copyright 2011 JG Electronics. All rights reserved.
//	Remarks, reports to: www.jgelectronics.nl.
//
//	Derived from Apple's example program AudioQueueTools.
//	UI and icons from MPlayerX an open source application by http://mplayerx.org
//
//	OSX10.6 and XCODE 4.0.1
//
/*
 Disclaimer: IMPORTANT: 
 This JG Electronics software is supplied to you by JG Electronics as FREE AND OPEN SOURCE SOFTWARE. 
 
 The JG Electronics Software is provided by JG Electronics on an "AS IS" basis.
 JG ELECTRONICS MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE JG ELECTRONICS SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL JG ELECTRONICS BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE JG ELECTRONICS SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF JG ELECTRONICS HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

#import "UIView.h"

#define CONTROLALPHA		(1)
#define BACKGROUNDALPHA		(0.9)
#define CONTROL_CORNER_RADIUS	(6)
#define NUMOFVOLUMEIMAGES		(3)
#define AUTOHIDETIMEINTERNAL	(3)
#define LASTSTOPPEDTIMERATIO	(100)
#define ASPECTRATIOBASE			(900)

NSString * const kFillScreenButtonImageLRKey	= @"LR";
NSString * const kFillScreenButtonImageUBKey	= @"UB";
NSString * const kStringFMTTimeAppendTotal		= @" / %@";
NSString * const kUDKeyCtrlUIBackGroundAlpha	= @"CtrlUIBackGroundAlpha";


@implementation UIView

	#pragma mark -
	#pragma mark === Internal Methods ===
//	*********************************************************************************************************************************
- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
		notifCenter = [NSNotificationCenter defaultCenter];
		
		shouldHide = NO;
		fillGradient = nil;
		backGroundColor = nil;
		backGroundColor2 = nil;
		autoHideTimer = nil;
		autoHideTimeInterval = 0;
//		timeFormatter = [[TimeFormatter alloc] init];
//		floatWrapFormatter = [[FloatWrapFormatter alloc] init];
	}
    
    return self;
}

-(void) dealloc {
	[notifCenter removeObserver:self];
	
	if (autoHideTimer) {
		[autoHideTimer invalidate];
	}
	
//	[timeFormatter release];
//	[floatWrapFormatter release];
	
	[fillGradient release];
	[backGroundColor release];
	[backGroundColor2 release];
	
	[super dealloc];
}

- (void)awakeFromNib {
	orgHeight = [self bounds].size.height;
	[self setAlphaValue:CONTROLALPHA];
	[self refreshBackgroundAlpha];
}

	#pragma mark -
	#pragma mark === View Methods ===
//	*********************************************************************************************************************************
-(BOOL) acceptsFirstMouse:(NSEvent *)event {
	return YES;
}

-(BOOL) acceptsFirstResponder {
	return YES;
}

-(void) refreshBackgroundAlpha {
	[fillGradient release];
	[backGroundColor release];
	[backGroundColor2 release];
	
//	float backAlpha = [ud floatForKey:kUDKeyCtrlUIBackGroundAlpha];
	float backAlpha = 1.0;
	
	fillGradient = [[NSGradient alloc] initWithColorsAndLocations:[NSColor colorWithDeviceWhite:0.220 alpha:backAlpha], 0.00f,
					[NSColor colorWithDeviceWhite:0.150 alpha:backAlpha], 0.30f,
					[NSColor colorWithDeviceWhite:0.090 alpha:backAlpha], 0.33f,
					[NSColor colorWithDeviceWhite:0.050 alpha:backAlpha], 1.00f,	
					nil];
	backGroundColor  = [[NSColor colorWithDeviceWhite:0.45 alpha:backAlpha] retain];
	backGroundColor2 = [[NSColor colorWithDeviceWhite:0.32 alpha:backAlpha] retain];
	
	[self setNeedsDisplay:YES];
}

	#pragma mark -
	#pragma mark === Drawing Methods ===
//	*********************************************************************************************************************************
- (void)drawRect:(NSRect)dirtyRect {
	NSLog(@"drawRect");
    NSRect rc = [self bounds];
	NSPoint pt;

//	main background
	NSBezierPath *fillPath = [NSBezierPath bezierPathWithRoundedRect:rc xRadius:CONTROL_CORNER_RADIUS yRadius:CONTROL_CORNER_RADIUS];
	[fillGradient drawInBezierPath:fillPath angle:270];
	
//	top line
	[backGroundColor set];
	NSBezierPath *hilightPath = [NSBezierPath bezierPath];
	
	pt.x = rc.size.width - CONTROL_CORNER_RADIUS;
	pt.y = rc.size.height;
	[hilightPath moveToPoint:pt];
	
	pt.x = CONTROL_CORNER_RADIUS;
	[hilightPath lineToPoint:pt];
	
	[hilightPath stroke];
	
//	round corner line
	[backGroundColor2 set];
	
	NSBezierPath *roundPath = [NSBezierPath bezierPath];
	pt.x = rc.size.width;
	pt.y = rc.size.height - CONTROL_CORNER_RADIUS;
	[roundPath moveToPoint:pt];
	
	pt.x = rc.size.width - CONTROL_CORNER_RADIUS;
	[roundPath appendBezierPathWithArcWithCenter:pt radius:CONTROL_CORNER_RADIUS
									  startAngle:0 endAngle:90];
	pt.x = CONTROL_CORNER_RADIUS;
	pt.y = rc.size.height;
	[roundPath moveToPoint:pt];
	
	pt.y = rc.size.height - CONTROL_CORNER_RADIUS;
	[roundPath appendBezierPathWithArcWithCenter:pt radius:CONTROL_CORNER_RADIUS
									  startAngle:90 endAngle:180];
	[roundPath stroke];
}

@end
